/* 
  ______________________________________________________ 
 |														|
 |  Datei: BT-120032.C									|
 |														|
 |  Ansteuerung eines Graphic LCD Modul					|
 |  BATRON BT-120032									|
 |  LCD-Controller SED1330FBA							|
 |  Textmode 4 Zeilen 20 Zeichen je Zeile,				|
 |  Wolfgang Brust										|
 |  9.01.2007											|
 |  Änderungen:											|
 |  24.11.2015 externer Zeichensatz mit Umlaute und		|
 |			   Sonderzeichen							|
 |														|
 |  23.12.2015 Umlaute rausfischen und ersetzen			|
 |______________________________________________________|
 
*/

#include "BT-120032.h"


void LCDCMD_Byte(unsigned char data)

{ 
  sbi_LCD_RS;
  LCD_PORT=(data);
  sbi_LCD_E;
  cbi_LCD_E;
  cbi_LCD_RS;
}

void LCDDAT_Byte(unsigned char data)

{
  cbi_LCD_RS;
  LCD_PORT=(data);
  sbi_LCD_E;
  cbi_LCD_E;
}


void lcd_write_command( unsigned char command, uint8_t p_len, unsigned char parameter[])
{
 uint8_t i;
 
 LCDCMD_Byte(command);
 if (p_len != 0) 
   {
	for (i = 0; i < p_len; i++) 
		LCDDAT_Byte(parameter[i]);
	}
}

//******************************************** 
// Löscht das Display
// Cursor steht dann auf Position 1,Zeile 1
//********************************************
void LCDCls (void)
{
 unsigned int i;
//
 LCDPos(1,1);					//cur_X, cur_Y
 LCDCMD_Byte(MWRITE);			
 
 for(i=0; i<80; i++)			//80=((VSIZE/VPIXELS)*(HSIZE/HPIXELS))
      LCDDAT_Byte(0x20);		//write blank
    
  LCDPos(1,1);
}
 
//***************************************************
// Springt auf Zeile y und Position x 
// Gültig für 4 Zeilen
//***************************************************
void LCDPos(unsigned char cur_Y,unsigned char cur_X)
{
unsigned int c;

 LCDCMD_Byte(CSRW);					
 c=((cur_Y-1)*20)+(cur_X-1);	//20=(HSIZE/HPIXELS)		
 LCDDAT_Byte((char)(c));		//CSRL
 LCDDAT_Byte((char)(c>>=8));	//CSRH
}
 
//**************************************
//Cursor on/off
//**************************************
void Cursor(unsigned char data)
{

   LCDCMD_Byte(DISP_ON); //command
   LCDDAT_Byte(data);
}

//***************************************
//Schreibt eine Byte
//***************************************
void LCDWriteByte(unsigned char data)
{
  LCDCMD_Byte(MWRITE); 
  LCDDAT_Byte(data);
}

//****************************************
//Schreibt einen String
//****************************************
void LCDWrite(char *__putstr)
{
 char __ps;
 
 LCDCMD_Byte(MWRITE); 
	while((__ps = *__putstr++)!=0)
	{
		switch (__ps) //Umlaute rausfischen (abhängig vom Textcodierung)
		{
			case 0xC3:break;// Skip
			case 0xBC:LCDDAT_Byte(CHAR1);break;//ü
			case 0xA4:LCDDAT_Byte(CHAR2);break;//ä
			case 0xB6:LCDDAT_Byte(CHAR3);break;//ö
			case 0x9F:LCDDAT_Byte(CHAR4);break;//ß	
			default: LCDDAT_Byte(__ps);break;	
		}
	}
} 

//*****************************************
//Schreibt einen String aus ProgMem
//*****************************************
void P_LCDWrite (PGM_P s) 
{
	char __ps;
	LCDCMD_Byte(MWRITE);
	while ((__ps = pgm_read_byte(s++)) != 0)
	{
		switch (__ps) //Umlaute rausfischen (abhängig vom Textcodierung)
		{
			case 0xC3:break;// Skip
			case 0xBC:LCDDAT_Byte(CHAR1);break;//ü
			case 0xA4:LCDDAT_Byte(CHAR2);break;//ä
			case 0xB6:LCDDAT_Byte(CHAR3);break;//ö
			case 0x9F:LCDDAT_Byte(CHAR4);break;//ß	
			default: LCDDAT_Byte(__ps);break;	
		}
	}
}

//*****************************************
//Zeigt ein Byte in HexCode
//*****************************************
void ShowHex(uint8_t data)
{
	uint8_t temp;
	
	LCDWrite(" 0x");
	temp=data >> 4;			//High nibble zuerst
	if(temp<10) temp+='0';	//ASCII Code erzeugen
	else temp+=0x37;        //Großbuchstaben
	LCDWriteByte(temp);
	
	temp=data & 0x0F;		//Danach das Low nibble
	if(temp<10) temp+='0';	//ASCII Code erzeugen
	else temp+=0x37;        //Großbuchstaben
	LCDWriteByte(temp);
}

//*****************************************
//Zeigt ein Byte in BinCode 
//*****************************************
void ShowBin(uint8_t data)
{
	uint8_t i,temp;
	
	for ( i=8; i>0; i--)		   
	{
		temp=((data>>(i-1)) & 0x01)+'0';//ASCII Code erzeugen 0/1
		LCDWriteByte(temp);	
	}
	
}

//******************************************************
//Stellt die Funktionsweise des Graphic LCD Modules ein
//4 Zeilen 20 Zeichen je Zeile
//******************************************************
void LCDInit(void)
{
unsigned char para[10];

 cbi_RESET;							//LCD_RESET
 _delay_us(250);				
 sbi_RESET;
_delay_ms(5);
 
 /*
 The pulsewidth on RESET must be at least 200 us. 
 Note that pulses of more than a few seconds will 
 cause DC voltages to be applied to the LCD panel.
*/

 // System Set
 //bei KS0086 SEGMENT DRIVER  para[3] = (HSIZE/HPIXELS + 2)
 para[0] = 0x30;					// P1 W/s, M2, M1, M0 
 para[1] = (WF*128 + (HPIXELS-1));	// P2 WF, FX
 para[2] = (VPIXELS-1);				// P3 FY
 para[3] = (HSIZE/HPIXELS - 1);   	// P4 C/R
 para[4] = SED_TCR;					// P5 TC/R
 para[5] = (VSIZE-1); 				// P6 L/F
 para[6] = (HSIZE/HPIXELS); 		// P7 APL
 para[7] = 0x00; 					// P8 APH
 lcd_write_command(SYSTEM_SET, 8, para);
 
 // Scroll
 para[0] = 0x00; 					// P1 SAD1L
 para[1] = 0x00; 					// P2 SAD1H
 para[2] = (VSIZE - 1); 			// P3 SL1
 para[3] = 0x00; 					// P4 SAD2L
 para[4] = 0x08; 					// P5 SAD2H
 para[5] = (VSIZE - 1); 			// P6 SL2
 para[6] = 0x00; 					// P7 SAD3L
 para[7] = 0x10; 					// P8 SAD3H
 para[8] = (VSIZE - 1); 			// P9 SL3
 lcd_write_command(SCROLL, 9, para);
 
 // HDOT SCR
 para[0] = 0x00;
 lcd_write_command(HDOT_SCR, 1, para);  
  
 // OVLAY Parameter
 para[0] = 0x00;
 lcd_write_command(OVLAY, 1, para);
 
 //Cursor Form
 para[0]= 0x04; 					// CRX
 para[1]= 0x06;  // CM, CRY Underline cursor (0x86 Block cursor)
 lcd_write_command(CSRFORM, 2, para);
	
 LCDCMD_Byte(DISP_OFF);
 LCDCMD_Byte(CSRDIR_RIGHT);
	
//*******************************************************************
//externer Zeichensatz mit Umlaute und Sonderzeichen aus eeprom laden   
//*******************************************************************	
	
#ifdef ext_Zeichensatz  //externer Zeichensatz
	
	para[0]= 0x00;						// CSRL
	para[1]= 0x10;						// CSRH
	lcd_write_command(CSRW, 2, para);	// CSRW (CG_RAM 0x1000)
	LCDCMD_Byte(MWRITE);
	
	uint16_t i;
	for (i=0;i<1024;i++) //Zeichensatz aus eeprom -> CG_RAM 
		{
			LCDDAT_Byte( eeprom_read_byte ((uint8_t *)(i)));
			_delay_us(50);
		}
	
	LCDCMD_Byte(SYSTEM_SET);
	LCDDAT_Byte(0x31);					//W/s, M2, M1, M0 
	LCDCls();
	_delay_ms(200);
	
 //End externer Zeichensatz 
	
#else	
	
	para[0]= 0x00; 					// CSRL
	para[1]= 0x00; 					// CSRH
	lcd_write_command(CSRW, 2, para); // CSRW,
	
#endif 
	
 LCDCMD_Byte(DISP_ON);
 LCDCls();
 Cursor(off);

//End init
}

