/* Datei: BT-120032.h

   Ansteuerung eines Graphic LCD Modules
   BATRON BT-120032
   LCD-Controller SED1330FBA
   Textmode 4 Zeilen 20 Zeichen je Zeile             
   Wolfgang Brust
   9.01.2007
   Änderungen:
   
 ___________________________________________________________
 .__    __.        ._________.
 |  \__/  |		   |		 |
 |     Px7|	 ----> |14 D7  L | 
 |     Px6|  ----> |13 D6  C | 
 |     Px5|  ----> |12 D5  D |
 |     Px4|  ----> |11 D4    |
 |     Px3|  ----> |10 D3  M |
 |     Px2|  ----> | 9 D2  o |  
 |     Px1|  ----> | 8 D1  d |
 |     Px0|  ----> | 7 D0  u |
 |        |        |       l |
 |     Px3|	 ----> |16/RESET |
 |	   Px2|	 ----> | 6 E     |
 |	   Px1|	 ----> | 5 R/W   |
 |	   Px0|  ----> | 4 RS    |
 |		  | .----> | 3 Vo    |
 | ATMega | |Vcc-> | 2     15|/CS <-GND
 |		  | |GND-> | 1	   17|->VEE Negative Voltage Output
 |________| |	   |_________|		
			|
			| Contrast Voltage Regulation               
 
 
 _________________________________________________________________________________
 
 LCDPos(y,x);		Springt auf Zeile y und Position x, Gültig für 4 Zeilen
 LCDCls();			Löscht das Display, Cursor steht dann auf Position 1, Zeile 1
 Cursor(on);		Cursor on
 Cursor(off);		Cursor off
 LCDWrite("Text");	Schreibt einen String
 P_LCDWrite(String);Schreibt einen String aus ProgMem
 LCDWriteByte(x);	Schreibt ein Datenbyte 
 ShowHex(x);		Zeigt ein Byte in HexCode an
 ShowBin(x);		Zeigt ein Byte in BinCode an MSB->LSB
 
 _________________________________________________________________________________
 
*/

#include <avr/io.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

#define ext_Zeichensatz  // Zeichensatz mit Umlaute und Sonderzeichen aus eeprom
						 // eeprom size >= 1024kByte!
#define CHAR1   5	//ü
#define CHAR2   2	//ä
#define CHAR3   3	//ö
#define CHAR4   16	//ß

#define on 0x05
#define off 0x04

// display
#define HSIZE 120  // display resolution horizontal
#define VSIZE  32  // display resolution vertical
#define HPIXELS 6  // horizontal pixels per char (max. 8)
#define VPIXELS 8  // vertical pixels per char (max. 16)
#define WF		1  // AC frame drive: 0=16-line, 1=2-frame
#define SED_TCR 0xFF // value for TC/R register of SED1330

// control
#define SYSTEM_SET	0x40
#define MWRITE		0x42
#define MREAD		0x43
#define SCROLL		0x44	
#define CSRW		0x46	 
#define CSRR		0x47	
#define HDOT_SCR	0x5a	
#define OVLAY		0x5b	
#define CGRAM_ADR	0x5c	
#define CSRFORM		0x5d	

// display control
#define SLEEP_IN	0x53
#define DISP_OFF	0x58
#define DISP_ON		0x59
#define CSRDIR_RIGHT 0x4c
#define CSRDIR_LEFT	0x4d
#define CSRDIR_UP	0x4e
#define CSRDIR_DOWN	0x4f


// Ports
#define LCD_PORT  PORTC		
#define LCD_Cont  PORTD
#define LCD_RESET PORTA	

#define LCD_E  7	// E  LCD Enable
#define LCD_RS 6	// RS LCD Register Select
#define RESET  7	// /R LCD Reset
					// R/W, /CS connect to GND

#define cbi_RESET  LCD_RESET &= ~(1<<RESET)
#define sbi_RESET  LCD_RESET |= (1<<RESET)
#define sbi_LCD_RS LCD_Cont  |= (1<<LCD_RS)
#define cbi_LCD_RS LCD_Cont  &= ~(1<<LCD_RS)
#define sbi_LCD_E  LCD_Cont  |= (1<<LCD_E)
#define cbi_LCD_E  LCD_Cont  &= ~(1<<LCD_E)

void LCDCMD_Byte(unsigned char);
void LCDDAT_Byte(unsigned char);
void LCDInit(void);
void LCDWriteByte(unsigned char);
void LCDWrite(char *);
void ShowBin(uint8_t);
void ShowHex(uint8_t);
void P_LCDWrite (PGM_P s);
void LCDCls(void);
void LCDPos(unsigned char, unsigned char);
void CURSOR(unsigned char);
void lcd_write_command( unsigned char, uint8_t, unsigned char[]);
