/* Datei: lcd.c

 Ansteuerung eines LCD Modul
 HD44780-Kompatibel
 im 4 Bit Daten Mode
 Author: <Wolfgang Brust>
 03.12.2006
 * Copyright:
 * License: 
 
 Last change: 
 23.12.2015 Umlaute rausfischen und ersetzen

*/

#include "lcd.h"


//*******************************************
// Schreibt ein Datenbyte 
//*******************************************
void LCDWriteByte(unsigned char data)
{
 unsigned char temp;
 
 temp=data;
 temp&=0xF0;		//High Nibble zuerst
 temp>>=2;			//D5..D2 !
 LCD_PORT&=0x03;	//D5..2 lschen
 LCD_PORT|=temp;
 SBI_CS;
 CLR_CS;	
 temp=data;
 temp<<=2;			//Low Nibble
 LCD_PORT&=0x03;	//D5..2 lschen
 LCD_PORT|=temp;
 SBI_CS;
 CLR_CS;	
 _delay_us(50);	//etwas auf die Anzeige warten (>39us fosc 270KHz)
}

//**************************************************
//Schreibt einen String
//**************************************************
void LCDWrite(char *__putstr)
{
	char __ps;
	
	while((__ps = *__putstr++)!=0)
	{
		switch (__ps) //Umlaute rausfischen  (abhngig vom Textcodierung)
		{
			case 0xC3:break;// Skip
			case 0xBC:LCDWriteByte(CHAR1);break;// "  "
			case 0xA4:LCDWriteByte(CHAR2);break;// "  "
			case 0xB6:LCDWriteByte(CHAR3);break;// "  "
			case 0x9F:LCDWriteByte(CHAR4);break;// "  "
			case 0x67:LCDWriteByte(CHAR5);break;// " g " sieht besser aus
			default: LCDWriteByte(__ps);break;	
		}
	}
}

//*****************************************
//Schreibt einen String aus ProgMem
//*****************************************
void P_LCDWrite (PGM_P s) 
{
	char __ps;
	while ((__ps = pgm_read_byte(s++)) != 0)
	{
		switch (__ps) //Umlaute rausfischen (abhngig vom Textcodierung)
		{
			case 0xC3:break;// Skip
			case 0xBC:LCDWriteByte(CHAR1);break;// "  "
			case 0xA4:LCDWriteByte(CHAR2);break;// "  "
			case 0xB6:LCDWriteByte(CHAR3);break;// "  "
			case 0x9F:LCDWriteByte(CHAR4);break;// "  "
			case 0x67:LCDWriteByte(CHAR5);break;// " g " sieht besser aus
			default: LCDWriteByte(__ps);break;	
		}
	}	
}


//****************************************************
// Springt auf Zeile y und Position x 
// Gltig fr 4 Zeilen
//****************************************************
void LCDPos(unsigned char zeile,unsigned char position)
{
 unsigned char temp;

 CLR_RS; //Umschalten auf Kontrollregister

 switch(zeile)
	{
		case 1: temp=0x7F+position; break;
		case 2: temp=0xBF+position; break;
		case 3: temp=0x93+position; break;
		case 4: temp=0xD3+position; break;
		default:temp=0x80; break; //1,1
	}

 LCDWriteByte(temp); 
  SBI_RS; //Umschalten auf Datenregister
 _delay_ms(2);
}

//*****************************************
// Lscht das Display
// Cursor steht dann auf Position 1,Zeile 1
//*****************************************
void LCDCls(void)
{
 CLR_RS; //Controlregister
 LCDWriteByte(0x01); //CLS
 SBI_RS; //Umschalten auf Datenregister
 _delay_ms(2);
}

//*****************************************
//Cursor on/off 
//*****************************************
void Cursor(unsigned char data)
{
	CLR_RS;
	LCDWriteByte(data);
	SBI_RS;
}
//*****************************************
//Zeigt ein Byte in HexCode an
//*****************************************
void ShowHex(uint8_t data)
{
	uint8_t temp;
	
	LCDWrite(" 0x");
	temp=data >> 4;			//High nibble zuerst
	if(temp<10) temp+='0';	//ASCII Code erzeugen
	else temp+=0x37;        //Grobuchstaben
	LCDWriteByte(temp);
	
	temp=data & 0x0F;		//Danach das Low nibble
	if(temp<10) temp+='0';	//ASCII Code erzeugen
	else temp+=0x37;        //Grobuchstaben
	LCDWriteByte(temp);
}
//*****************************************
//Zeigt ein Byte in BinCode an 
//*****************************************
void ShowBin(uint8_t data)
{
	uint8_t i,temp;
	
	for ( i=8; i>0; i--)		   
	{
		temp=((data>>(i-1)) & 0x01)+'0';//ASCII Code erzeugen 0/1
		LCDWriteByte(temp);	
	}
	
}

//*********************************************
//Schreibt das CG-RAM
//*********************************************
void Generatechar(unsigned char Adress, char *data )
{	
	unsigned char i;
	CLR_RS;
    LCDWriteByte(CGADR | (Adress<<3) );// StartAdresse des Zeichens einstellen
	SBI_RS;
	_delay_us(50);
    for ( i=0; i<8; i++ )
		{
			LCDWriteByte( data[i] );// Bitmuster bertragen
			_delay_us(50);
		}
}

//*********************************************
//Stellt die Funktionsweise des LCD Display ein
//*********************************************
void LCDInit(void)
{
 unsigned char i;

 CLR_RS;  //Control Register
 CLR_CS;  //LCD-Anzeige 

//Software Reset
 _delay_ms(100);    //Power On Reset abwarten
 for(i=0; i<3; i++)
	{
		LCD_PORT=0x30;	
		SBI_CS;	
		CLR_CS; 
		_delay_ms(100); //etwas warten auf die Anzeige
	}
//4Bit Operation 
 LCD_PORT=0x20;	   //High Nibble schreiben
 SBI_CS;	
 CLR_CS; 
 _delay_ms(5);//5
	
 LCDWriteByte(0x28);    //4 Bit Operation,2 Line,5x7 Font
//LCDWriteByte(0x38);   //8 Bit Operation,2 Line,5x7 Font
 _delay_ms(2);
 LCDWriteByte(0x14); 	//Cursor Move,Right Shift
 _delay_ms(2);

// LCDWriteByte(0x0F); 	//Display on, Cursor on ,Cursor Blinking
// LCDWriteByte(0x0E);  //Display on, Cursor on ,Cursor no Blinking
 LCDWriteByte(0x0C); 	//Display on, Cursor off ,Cursor no Blinking
 _delay_ms(2);

 LCDWriteByte(0x06); 	//Increment, Display 
 _delay_ms(2);
 LCDWriteByte(0x02); 	//Cursor Home
 _delay_ms(2);
	
  SBI_RS;				//LCD-Datenregister
	

#ifdef Sonderzeichen	//CHAR_Generator  8 CHR
	char buff[8];

	//*CHAR1*
	buff[0]=0b00001010;	// CHAR0 "  "
	buff[1]=0b00000000;						 
	buff[2]=0b00010001;
	buff[3]=0b00010001;			
	buff[4]=0b00010001;
	buff[5]=0b00010011;
	buff[6]=0b00001101;
	buff[7]=0b00000000;	// Cursor line
	
	Generatechar(1,buff);  // (0...7,temp)
	
	
	//*CHAR2*				    
	buff[0]=0b00001010;	// CHAR2 "  "
	buff[1]=0b00000000;						 
	buff[2]=0b00001110;
	buff[3]=0b00000001;			
	buff[4]=0b00001111;
	buff[5]=0b00010001;
	buff[6]=0b00001111;
	buff[7]=0b00000000;	// Cursor line
	
	Generatechar(2,buff);// 0...7,temp

	
	//*CHAR3*
	buff[0]=0b00001010;	// CHAR3 "  "
	buff[1]=0b00000000;						 
	buff[2]=0b00001110;
	buff[3]=0b00010001;			
	buff[4]=0b00010001;
	buff[5]=0b00010001;
	buff[6]=0b00001110;
	buff[7]=0b00000000;	// Cursor line
	
	Generatechar(3,buff);// (0...7,temp)
	
	//*CHAR4*
	buff[0]=0b00001110;	// CHAR4 "  "
	buff[1]=0b00010001;	
	buff[2]=0b00011110;
	buff[3]=0b00010001;	 
	buff[4]=0b00011001;
	buff[5]=0b00010110;
	buff[6]=0b00010000;
	buff[7]=0b00010000;// Cursor line
	
	Generatechar(4,buff);// (0...7,temp)
	
	//*CHAR5*
	buff[0]=0b00000000;	// CHAR5 " g " sieht besser aus
	buff[1]=0b00000000;	
	buff[2]=0b00001111;
	buff[3]=0b00010001;	 
	buff[4]=0b00010001;
	buff[5]=0b00001111;
	buff[6]=0b00000001;
	buff[7]=0b00001110;// Cursor line
	
	Generatechar(5,buff);// (0...7,temp)
	
	/*CHAR*
	buff[0]=0b00000000;	//
	buff[1]=0b00000000;	
	buff[2]=0b00000000;
	buff[3]=0b00000000;	 
	buff[4]=0b00000000;
	buff[5]=0b00000000;
	buff[6]=0b00000000;
	buff[7]=0b00000000;// Cursor line
	
	Generatechar(x,buff);// (0...7,temp)
	*/
#endif
	
}


