/* Datei: lcd.h
 
 Ansteuerung eines LCD Modul
 im 4 Bit Daten Mode
 Author: <Wolfgang Brust>
 * Copyright:
 * License: 
 
 Last change: 03.12.06
 Compiler AVR-GCC
 
 LCD Modul HD44780-KOMPATIBEL
 Ansteuerung des LCD-Modul
 LCD_PORT (Px2-Px5) an den Datenleitungen
 Px0 an E, Px1 an RS, R/W auf Masse (Gnd)
 
	    .__    __.        ._________.
        |  \__/  |		  |		    |
        |        |		  |		    |
        |     Px5|  ----> |14 D7  L |
        |     Px4|  ----> |13 D6  C |
        |     Px3|  ----> |12 D5  D |
        |     Px2|  ----> |11 D4    |
        |     Px1|  ----> | 4 RS  M |
        |     Px0|  ----> | 6 E   o |
        |        |  GND-> | 5 R/W d |
        |        | .--->  |	3 Vo  u |
	    | ATMega | |Vcc-> |	2	  l |
        | ATtiny | |GND-> |	1	    |
        |________| |	  |_________|		
				   |
				   | Kontrast einstellung
 ___________________________________________________________
 
 LCDPos(y,x);		Springt auf Zeile y und Position x, Gltig fr 4 Zeilen
 LCDCls();			Lscht das Display, Cursor steht dann auf Position 1, Zeile 1
 Cursor(on);		Cursor on
 Cursor(off);		Cursor off
 LCDWrite("Text");	Schreibt einen String
 P_LCDWrite(String);Schreibt einen String aus ProgMem
 LCDWriteByte(x);	Schreibt ein Datenbyte 
 ShowHex(x);		Zeigt ein Byte in HexCode an
 ShowBin(x);		Zeigt ein Byte in BinCode an MSB->LSB
 ___________________________________________________________
*/

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#define Sonderzeichen //Umlaute und Sonderzeichen
#define CGADR	0x40  

#define CHAR1   1	//
#define CHAR2   2	//
#define CHAR3   3	//
#define CHAR4   4	//
#define CHAR5   5	//g sieht besser aus

#define on 0xE		//Display on, Cursor on,  blink off
#define off 0xC		//Display on, Cursor off, blink off
//
#define LCD_PORT PORTC	//Portx 0-5

#define LCD_CS	0	//LCD Chip Select	   (E)
#define LCD_RS	1	//LCD Register Select (RS)
//
#define SBI_CS  LCD_PORT |= (1<<LCD_CS)
#define CLR_CS  LCD_PORT &= ~(1<<LCD_CS)
#define SBI_RS  LCD_PORT |= (1<<LCD_RS)  //Data Register
#define CLR_RS  LCD_PORT &= ~(1<<LCD_RS) //Instruction Register

void ShowHex(uint8_t);
void ShowBin(uint8_t);
void LCDInit(void);
void LCDWriteByte(unsigned char);
void LCDWrite(char *);
void LCDCls(void);
void LCDPos(unsigned char, unsigned char);
void Cursor(unsigned char);
void P_LCDWrite (PGM_P s);
void ShowHex(uint8_t);
void ShowBin(uint8_t);
void _Minuten (void);
